@@ -41,7 +41,7 @@ import (
 
 	networkv1 "github.com/k8snetworkplumbingwg/network-attachment-definition-client/pkg/apis/k8s.cni.cncf.io/v1"
 
-	v1 "kubevirt.io/client-go/api/v1"
+	v1 "kubevirt.io/api/core/v1"
 	"kubevirt.io/client-go/kubecli"
 	"kubevirt.io/client-go/log"
 	"kubevirt.io/client-go/precond"
@@ -57,9 +57,19 @@ import (
 	"kubevirt.io/kubevirt/pkg/virt-launcher/virtwrap/api"
 )
 
+const (
+	containerDisks   = "container-disks"
+	hotplugDisks     = "hotplug-disks"
+	hookSidecarSocks = "hook-sidecar-sockets"
+	varRun           = "/var/run"
+	virtBinDir       = "virt-bin-share-dir"
+	hotplugDisk      = "hotplug-disk"
+)
+
 const KvmDevice = "devices.kubevirt.io/kvm"
 const TunDevice = "devices.kubevirt.io/tun"
 const VhostNetDevice = "devices.kubevirt.io/vhost-net"
+const SevDevice = "devices.kubevirt.io/sev"
 
 const debugLogs = "debugLogs"
 const logVerbosity = "logVerbosity"
@@ -67,7 +77,6 @@ const virtiofsDebugLogs = "virtiofsdDebugLogs"
 
 const MultusNetworksAnnotation = "k8s.v1.cni.cncf.io/networks"
 
-const qemuTimeoutBaseSeconds = 240
 const qemuTimeoutJitterRange = 120
 
 const (
@@ -94,6 +103,11 @@ const MULTUS_DEFAULT_NETWORK_CNI_ANNOTATION = "v1.multus-cni.io/default-network"
 // Istio list of virtual interfaces whose inbound traffic (from VM) will be treated as outbound traffic in envoy
 const ISTIO_KUBEVIRT_ANNOTATION = "traffic.sidecar.istio.io/kubevirtInterfaces"
 
+const VELERO_PREBACKUP_HOOK_CONTAINER_ANNOTATION = "pre.hook.backup.velero.io/container"
+const VELERO_PREBACKUP_HOOK_COMMAND_ANNOTATION = "pre.hook.backup.velero.io/command"
+const VELERO_POSTBACKUP_HOOK_CONTAINER_ANNOTATION = "post.hook.backup.velero.io/container"
+const VELERO_POSTBACKUP_HOOK_COMMAND_ANNOTATION = "post.hook.backup.velero.io/command"
+
 const ENV_VAR_LIBVIRT_DEBUG_LOGS = "LIBVIRT_DEBUG_LOGS"
 const ENV_VAR_VIRTIOFSD_DEBUG_LOGS = "VIRTIOFSD_DEBUG_LOGS"
 const ENV_VAR_VIRT_LAUNCHER_LOG_VERBOSITY = "VIRT_LAUNCHER_LOG_VERBOSITY"
@@ -118,6 +132,7 @@ type TemplateService interface {
 
 type templateService struct {
 	launcherImage              string
+	launcherQemuTimeout        int
 	virtShareDir               string
 	virtLibDir                 string
 	ephemeralDiskDir           string
@@ -130,7 +145,21 @@ type templateService struct {
 	launcherSubGid             int64
 }
 
-type PvcNotFoundError error
+type PvcNotFoundError struct {
+	Reason string
+}
+
+func (e PvcNotFoundError) Error() string {
+	return e.Reason
+}
+
+type DataVolumeNotFoundError struct {
+	Reason string
+}
+
+func (e DataVolumeNotFoundError) Error() string {
+	return e.Reason
+}
 
 func isFeatureStateEnabled(fs *v1.FeatureState) bool {
 	return fs != nil && fs.Enabled != nil && *fs.Enabled
@@ -394,12 +423,38 @@ func (t *templateService) IsARM64() bool {
 	return t.clusterConfig.GetClusterCPUArch() == "arm64"
 }
 
-func generateQemuTimeoutWithJitter() string {
+func generateQemuTimeoutWithJitter(qemuTimeoutBaseSeconds int) string {
 	timeout := rand.Intn(qemuTimeoutJitterRange) + qemuTimeoutBaseSeconds
 
 	return fmt.Sprintf("%ds", timeout)
 }
 
+func (t *templateService) addPVCToLaunchManifest(volume v1.Volume, claimName string, namespace string, volumeMounts *[]k8sv1.VolumeMount, volumeDevices *[]k8sv1.VolumeDevice) error {
+	logger := log.DefaultLogger()
+	_, exists, isBlock, err := types.IsPVCBlockFromStore(t.persistentVolumeClaimStore, namespace, claimName)
+	if err != nil {
+		logger.Errorf("error getting PVC: %v", claimName)
+		return err
+	} else if !exists {
+		logger.Errorf("didn't find PVC %v", claimName)
+		return PvcNotFoundError{Reason: fmt.Sprintf("didn't find PVC %v", claimName)}
+	} else if isBlock {
+		devicePath := filepath.Join(string(filepath.Separator), "dev", volume.Name)
+		device := k8sv1.VolumeDevice{
+			Name:       volume.Name,
+			DevicePath: devicePath,
+		}
+		*volumeDevices = append(*volumeDevices, device)
+	} else {
+		volumeMount := k8sv1.VolumeMount{
+			Name:      volume.Name,
+			MountPath: hostdisk.GetMountedHostDiskDir(volume.Name),
+		}
+		*volumeMounts = append(*volumeMounts, volumeMount)
+	}
+	return nil
+}
+
 func (t *templateService) renderLaunchManifest(vmi *v1.VirtualMachineInstance, imageIDs map[string]string, tempPod bool) (*k8sv1.Pod, error) {
 	precond.MustNotBeNil(vmi)
 	domain := precond.MustNotBeEmpty(vmi.GetObjectMeta().GetName())
@@ -447,6 +502,12 @@ func (t *templateService) renderLaunchManifest(vmi *v1.VirtualMachineInstance, i
 			hotplugVolumes[volumeStatus.Name] = true
 		}
 	}
+	// This detects hotplug volumes for a started but not ready VMI
+	for _, volume := range vmi.Spec.Volumes {
+		if (volume.DataVolume != nil && volume.DataVolume.Hotpluggable) || (volume.PersistentVolumeClaim != nil && volume.PersistentVolumeClaim.Hotpluggable) {
+			hotplugVolumes[volume.Name] = true
+		}
+	}
 
 	// Need to run in privileged mode in Power or libvirt will fail to lock memory for VMI
 	if t.IsPPC64() {
@@ -465,13 +526,13 @@ func (t *templateService) renderLaunchManifest(vmi *v1.VirtualMachineInstance, i
 
 	prop := k8sv1.MountPropagationHostToContainer
 	volumeMounts = append(volumeMounts, k8sv1.VolumeMount{
-		Name:             "container-disks",
+		Name:             containerDisks,
 		MountPath:        t.containerDiskDir,
 		MountPropagation: &prop,
 	})
 	if !vmi.Spec.Domain.Devices.DisableHotplug {
 		volumeMounts = append(volumeMounts, k8sv1.VolumeMount{
-			Name:             "hotplug-disks",
+			Name:             hotplugDisks,
 			MountPath:        t.hotplugDiskDir,
 			MountPropagation: &prop,
 		})
@@ -500,39 +561,26 @@ func (t *templateService) renderLaunchManifest(vmi *v1.VirtualMachineInstance, i
 		if hotplugVolumes[volume.Name] {
 			continue
 		}
-		volumeMount := k8sv1.VolumeMount{
-			Name:      volume.Name,
-			MountPath: hostdisk.GetMountedHostDiskDir(volume.Name),
-		}
 		if volume.PersistentVolumeClaim != nil {
-			logger := log.DefaultLogger()
 			claimName := volume.PersistentVolumeClaim.ClaimName
-			_, exists, isBlock, err := types.IsPVCBlockFromStore(t.persistentVolumeClaimStore, namespace, claimName)
-			if err != nil {
-				logger.Errorf("error getting PVC: %v", claimName)
+			if err := t.addPVCToLaunchManifest(volume, claimName, namespace, &volumeMounts, &volumeDevices); err != nil {
 				return nil, err
-			} else if !exists {
-				logger.Errorf("didn't find PVC %v", claimName)
-				return nil, PvcNotFoundError(fmt.Errorf("didn't find PVC %v", claimName))
-			} else if isBlock {
-				devicePath := filepath.Join(string(filepath.Separator), "dev", volume.Name)
-				device := k8sv1.VolumeDevice{
-					Name:       volume.Name,
-					DevicePath: devicePath,
-				}
-				volumeDevices = append(volumeDevices, device)
-			} else {
-				volumeMounts = append(volumeMounts, volumeMount)
 			}
 			volumes = append(volumes, k8sv1.Volume{
 				Name: volume.Name,
 				VolumeSource: k8sv1.VolumeSource{
-					PersistentVolumeClaim: volume.PersistentVolumeClaim,
+					PersistentVolumeClaim: &k8sv1.PersistentVolumeClaimVolumeSource{
+						ClaimName: volume.PersistentVolumeClaim.ClaimName,
+						ReadOnly:  volume.PersistentVolumeClaim.ReadOnly,
+					},
 				},
 			})
 		}
 		if volume.Ephemeral != nil {
-			volumeMounts = append(volumeMounts, volumeMount)
+			claimName := volume.Ephemeral.PersistentVolumeClaim.ClaimName
+			if err := t.addPVCToLaunchManifest(volume, claimName, namespace, &volumeMounts, &volumeDevices); err != nil {
+				return nil, err
+			}
 			volumes = append(volumes, k8sv1.Volume{
 				Name: volume.Name,
 				VolumeSource: k8sv1.VolumeSource{
@@ -570,26 +618,10 @@ func (t *templateService) renderLaunchManifest(vmi *v1.VirtualMachineInstance, i
 			})
 		}
 		if volume.DataVolume != nil {
-			logger := log.DefaultLogger()
 			claimName := volume.DataVolume.Name
-			_, exists, isBlock, err := types.IsPVCBlockFromStore(t.persistentVolumeClaimStore, namespace, claimName)
-			if err != nil {
-				logger.Errorf("error getting PVC associated with DataVolume: %v", claimName)
+			if err := t.addPVCToLaunchManifest(volume, claimName, namespace, &volumeMounts, &volumeDevices); err != nil {
 				return nil, err
-			} else if !exists {
-				logger.Errorf("didn't find PVC associated with DataVolume: %v", claimName)
-				return nil, PvcNotFoundError(fmt.Errorf("didn't find PVC associated with DataVolume: %v", claimName))
-			} else if isBlock {
-				devicePath := filepath.Join(string(filepath.Separator), "dev", volume.Name)
-				device := k8sv1.VolumeDevice{
-					Name:       volume.Name,
-					DevicePath: devicePath,
-				}
-				volumeDevices = append(volumeDevices, device)
-			} else {
-				volumeMounts = append(volumeMounts, volumeMount)
 			}
-
 			volumes = append(volumes, k8sv1.Volume{
 				Name: volume.Name,
 				VolumeSource: k8sv1.VolumeSource{
@@ -835,7 +867,7 @@ func (t *templateService) renderLaunchManifest(vmi *v1.VirtualMachineInstance, i
 	gracePeriodKillAfter := gracePeriodSeconds + int64(15)
 
 	// Get memory overhead
-	memoryOverhead := getMemoryOverhead(vmi, t.clusterConfig.GetClusterCPUArch())
+	memoryOverhead := GetMemoryOverhead(vmi, t.clusterConfig.GetClusterCPUArch())
 
 	// Consider CPU and memory requests and limits for pod scheduling
 	resources := k8sv1.ResourceRequirements{}
@@ -959,13 +991,13 @@ func (t *templateService) renderLaunchManifest(vmi *v1.VirtualMachineInstance, i
 
 	if len(requestedHookSidecarList) != 0 {
 		volumes = append(volumes, k8sv1.Volume{
-			Name: "hook-sidecar-sockets",
+			Name: hookSidecarSocks,
 			VolumeSource: k8sv1.VolumeSource{
 				EmptyDir: &k8sv1.EmptyDirVolumeSource{},
 			},
 		})
 		volumeMounts = append(volumeMounts, k8sv1.VolumeMount{
-			Name:      "hook-sidecar-sockets",
+			Name:      hookSidecarSocks,
 			MountPath: hooks.HookSocketsSharedDirectory,
 		})
 	}
@@ -1012,7 +1044,7 @@ func (t *templateService) renderLaunchManifest(vmi *v1.VirtualMachineInstance, i
 			"echo", "bound PVCs"}
 	} else {
 		command = []string{"/usr/bin/virt-launcher",
-			"--qemu-timeout", generateQemuTimeoutWithJitter(),
+			"--qemu-timeout", generateQemuTimeoutWithJitter(t.launcherQemuTimeout),
 			"--name", domain,
 			"--uid", string(vmi.UID),
 			"--namespace", namespace,
@@ -1028,20 +1060,20 @@ func (t *templateService) renderLaunchManifest(vmi *v1.VirtualMachineInstance, i
 		}
 	}
 
-	useEmulation := t.clusterConfig.IsUseEmulation()
+	allowEmulation := t.clusterConfig.AllowEmulation()
 	imagePullPolicy := t.clusterConfig.GetImagePullPolicy()
 
 	if resources.Limits == nil {
 		resources.Limits = make(k8sv1.ResourceList)
 	}
 
-	extraResources := getRequiredResources(vmi, useEmulation)
+	extraResources := getRequiredResources(vmi, allowEmulation)
 	for key, val := range extraResources {
 		resources.Limits[key] = val
 	}
 
-	if useEmulation {
-		command = append(command, "--use-emulation")
+	if allowEmulation {
+		command = append(command, "--allow-emulation")
 	} else {
 		resources.Limits[KvmDevice] = resource.MustParse("1")
 	}
@@ -1072,6 +1104,11 @@ func (t *templateService) renderLaunchManifest(vmi *v1.VirtualMachineInstance, i
 		}
 	}
 
+	err = validatePermittedHostDevices(&vmi.Spec, t.clusterConfig)
+	if err != nil {
+		return nil, err
+	}
+
 	if util.IsGPUVMI(vmi) {
 		for _, gpu := range vmi.Spec.Domain.Devices.GPUs {
 			requestResource(&resources, gpu.DeviceName)
@@ -1084,6 +1121,10 @@ func (t *templateService) renderLaunchManifest(vmi *v1.VirtualMachineInstance, i
 		}
 	}
 
+	if util.IsSEVVMI(vmi) {
+		requestResource(&resources, SevDevice)
+	}
+
 	// VirtualMachineInstance target container
 	compute := k8sv1.Container{
 		Name:            "compute",
@@ -1109,15 +1150,15 @@ func (t *templateService) renderLaunchManifest(vmi *v1.VirtualMachineInstance, i
 		compute.Env = append(compute.Env,
 			k8sv1.EnvVar{
 				Name:  "XDG_CACHE_HOME",
-				Value: "/var/run",
+				Value: varRun,
 			},
 			k8sv1.EnvVar{
 				Name:  "XDG_CONFIG_HOME",
-				Value: "/var/run",
+				Value: varRun,
 			},
 			k8sv1.EnvVar{
 				Name:  "XDG_RUNTIME_DIR",
-				Value: "/var/run",
+				Value: varRun,
 			},
 		)
 	}
@@ -1169,10 +1210,10 @@ func (t *templateService) renderLaunchManifest(vmi *v1.VirtualMachineInstance, i
 	// Make sure the compute container is always the first since the mutating webhook shipped with the sriov operator
 	// for adding the requested resources to the pod will add them to the first container of the list
 	containers := []k8sv1.Container{compute}
-	containersDisks := containerdisk.GenerateContainers(vmi, imageIDs, "container-disks", "virt-bin-share-dir")
+	containersDisks := containerdisk.GenerateContainers(vmi, imageIDs, containerDisks, virtBinDir)
 	containers = append(containers, containersDisks...)
 
-	kernelBootContainer := containerdisk.GenerateKernelBootContainer(vmi, imageIDs, "container-disks", "virt-bin-share-dir")
+	kernelBootContainer := containerdisk.GenerateKernelBootContainer(vmi, imageIDs, containerDisks, virtBinDir)
 	if kernelBootContainer != nil {
 		log.Log.Object(vmi).Infof("kernel boot container generated")
 		containers = append(containers, *kernelBootContainer)
@@ -1180,7 +1221,7 @@ func (t *templateService) renderLaunchManifest(vmi *v1.VirtualMachineInstance, i
 
 	volumes = append(volumes,
 		k8sv1.Volume{
-			Name: "virt-bin-share-dir",
+			Name: virtBinDir,
 			VolumeSource: k8sv1.VolumeSource{
 				EmptyDir: &k8sv1.EmptyDirVolumeSource{},
 			},
@@ -1199,14 +1240,14 @@ func (t *templateService) renderLaunchManifest(vmi *v1.VirtualMachineInstance, i
 		},
 	})
 	volumes = append(volumes, k8sv1.Volume{
-		Name: "container-disks",
+		Name: containerDisks,
 		VolumeSource: k8sv1.VolumeSource{
 			EmptyDir: &k8sv1.EmptyDirVolumeSource{},
 		},
 	})
 	if !vmi.Spec.Domain.Devices.DisableHotplug {
 		volumes = append(volumes, k8sv1.Volume{
-			Name: "hotplug-disks",
+			Name: hotplugDisks,
 			VolumeSource: k8sv1.VolumeSource{
 				EmptyDir: &k8sv1.EmptyDirVolumeSource{},
 			},
@@ -1276,7 +1317,7 @@ func (t *templateService) renderLaunchManifest(vmi *v1.VirtualMachineInstance, i
 			},
 			VolumeMounts: []k8sv1.VolumeMount{
 				{
-					Name:      "hook-sidecar-sockets",
+					Name:      hookSidecarSocks,
 					MountPath: hooks.HookSocketsSharedDirectory,
 				},
 			},
@@ -1305,7 +1346,7 @@ func (t *templateService) renderLaunchManifest(vmi *v1.VirtualMachineInstance, i
 
 		initContainerVolumeMounts := []k8sv1.VolumeMount{
 			{
-				Name:      "virt-bin-share-dir",
+				Name:      virtBinDir,
 				MountPath: "/init/usr/bin",
 			},
 		}
@@ -1350,14 +1391,20 @@ func (t *templateService) renderLaunchManifest(vmi *v1.VirtualMachineInstance, i
 		initContainers = append(initContainers, cpInitContainer)
 
 		// this causes containerDisks to be pre-pulled before virt-launcher starts.
-		initContainers = append(initContainers, containerdisk.GenerateInitContainers(vmi, imageIDs, "container-disks", "virt-bin-share-dir")...)
+		initContainers = append(initContainers, containerdisk.GenerateInitContainers(vmi, imageIDs, containerDisks, virtBinDir)...)
 
-		kernelBootInitContainer := containerdisk.GenerateKernelBootInitContainer(vmi, imageIDs, "container-disks", "virt-bin-share-dir")
+		kernelBootInitContainer := containerdisk.GenerateKernelBootInitContainer(vmi, imageIDs, containerDisks, virtBinDir)
 		if kernelBootInitContainer != nil {
 			initContainers = append(initContainers, *kernelBootInitContainer)
 		}
 	}
 
+	readinessGates := []k8sv1.PodReadinessGate{
+		{
+			ConditionType: v1.VirtualMachineUnpaused,
+		},
+	}
+
 	// TODO use constants for podLabels
 	pod := k8sv1.Pod{
 		ObjectMeta: metav1.ObjectMeta{
@@ -1383,6 +1430,7 @@ func (t *templateService) renderLaunchManifest(vmi *v1.VirtualMachineInstance, i
 			ImagePullSecrets:              imagePullSecrets,
 			DNSConfig:                     vmi.Spec.DNSConfig,
 			DNSPolicy:                     vmi.Spec.DNSPolicy,
+			ReadinessGates:                readinessGates,
 		},
 	}
 
@@ -1440,6 +1488,37 @@ func (t *templateService) renderLaunchManifest(vmi *v1.VirtualMachineInstance, i
 	return &pod, nil
 }
 
+func validatePermittedHostDevices(spec *v1.VirtualMachineInstanceSpec, config *virtconfig.ClusterConfig) error {
+	errors := make([]string, 0)
+
+	if hostDevs := config.GetPermittedHostDevices(); hostDevs != nil {
+		// build a map of all permitted host devices
+		supportedHostDevicesMap := make(map[string]bool)
+		for _, dev := range hostDevs.PciHostDevices {
+			supportedHostDevicesMap[dev.ResourceName] = true
+		}
+		for _, dev := range hostDevs.MediatedDevices {
+			supportedHostDevicesMap[dev.ResourceName] = true
+		}
+		for _, hostDev := range spec.Domain.Devices.GPUs {
+			if _, exist := supportedHostDevicesMap[hostDev.DeviceName]; !exist {
+				errors = append(errors, fmt.Sprintf("GPU %s is not permitted in permittedHostDevices configuration", hostDev.DeviceName))
+			}
+		}
+		for _, hostDev := range spec.Domain.Devices.HostDevices {
+			if _, exist := supportedHostDevicesMap[hostDev.DeviceName]; !exist {
+				errors = append(errors, fmt.Sprintf("HostDevice %s is not permitted in permittedHostDevices configuration", hostDev.DeviceName))
+			}
+		}
+	}
+
+	if len(errors) != 0 {
+		return fmt.Errorf(strings.Join(errors, " "))
+	}
+
+	return nil
+}
+
 func (t *templateService) RenderHotplugAttachmentPodTemplate(volumes []*v1.Volume, ownerPod *k8sv1.Pod, vmi *v1.VirtualMachineInstance, claimMap map[string]*k8sv1.PersistentVolumeClaim, tempPod bool) (*k8sv1.Pod, error) {
 	zero := int64(0)
 	sharedMount := k8sv1.MountPropagationHostToContainer
@@ -1456,13 +1535,13 @@ func (t *templateService) RenderHotplugAttachmentPodTemplate(volumes []*v1.Volum
 				}),
 			},
 			Labels: map[string]string{
-				v1.AppLabel: "hotplug-disk",
+				v1.AppLabel: hotplugDisk,
 			},
 		},
 		Spec: k8sv1.PodSpec{
 			Containers: []k8sv1.Container{
 				{
-					Name:    "hotplug-disk",
+					Name:    hotplugDisk,
 					Image:   t.launcherImage,
 					Command: command,
 					Resources: k8sv1.ResourceRequirements{ //Took the request and limits from containerDisk init container.
@@ -1483,7 +1562,7 @@ func (t *templateService) RenderHotplugAttachmentPodTemplate(volumes []*v1.Volum
 					},
 					VolumeMounts: []k8sv1.VolumeMount{
 						{
-							Name:             "hotplug-disks",
+							Name:             hotplugDisks,
 							MountPath:        "/path",
 							MountPropagation: &sharedMount,
 						},
@@ -1509,13 +1588,12 @@ func (t *templateService) RenderHotplugAttachmentPodTemplate(volumes []*v1.Volum
 			},
 			Volumes: []k8sv1.Volume{
 				{
-					Name: "hotplug-disks",
+					Name: hotplugDisks,
 					VolumeSource: k8sv1.VolumeSource{
 						EmptyDir: &k8sv1.EmptyDirVolumeSource{},
 					},
 				},
 			},
-			HostNetwork:                   true,
 			TerminationGracePeriodSeconds: &zero,
 		},
 	}
@@ -1596,14 +1674,14 @@ func (t *templateService) RenderHotplugAttachmentTriggerPodTemplate(volume *v1.V
 				}),
 			},
 			Labels: map[string]string{
-				v1.AppLabel: "hotplug-disk",
+				v1.AppLabel: hotplugDisk,
 			},
 			Annotations: annotationsList,
 		},
 		Spec: k8sv1.PodSpec{
 			Containers: []k8sv1.Container{
 				{
-					Name:    "hotplug-disk",
+					Name:    hotplugDisk,
 					Image:   t.launcherImage,
 					Command: command,
 					Resources: k8sv1.ResourceRequirements{ //Took the request and limits from containerDisk init container.
@@ -1624,7 +1702,7 @@ func (t *templateService) RenderHotplugAttachmentTriggerPodTemplate(volume *v1.V
 					},
 					VolumeMounts: []k8sv1.VolumeMount{
 						{
-							Name:             "hotplug-disks",
+							Name:             hotplugDisks,
 							MountPath:        "/path",
 							MountPropagation: &sharedMount,
 						},
@@ -1654,13 +1732,12 @@ func (t *templateService) RenderHotplugAttachmentTriggerPodTemplate(volume *v1.V
 					},
 				},
 				{
-					Name: "hotplug-disks",
+					Name: hotplugDisks,
 					VolumeSource: k8sv1.VolumeSource{
 						EmptyDir: &k8sv1.EmptyDirVolumeSource{},
 					},
 				},
 			},
-			HostNetwork:                   true,
 			TerminationGracePeriodSeconds: &zero,
 		},
 	}
@@ -1730,19 +1807,18 @@ func getRequiredCapabilities(vmi *v1.VirtualMachineInstance, config *virtconfig.
 		capabilities = append(capabilities, CAP_SYS_ADMIN)
 		capabilities = append(capabilities, getVirtiofsCapabilities()...)
 	}
-
 	return capabilities
 }
 
-func getRequiredResources(vmi *v1.VirtualMachineInstance, useEmulation bool) k8sv1.ResourceList {
+func getRequiredResources(vmi *v1.VirtualMachineInstance, allowEmulation bool) k8sv1.ResourceList {
 	res := k8sv1.ResourceList{}
 	if (len(vmi.Spec.Domain.Devices.Interfaces) > 0) ||
 		(vmi.Spec.Domain.Devices.AutoattachPodInterface == nil) ||
 		(*vmi.Spec.Domain.Devices.AutoattachPodInterface == true) {
 		res[TunDevice] = resource.MustParse("1")
 	}
-	if util.NeedVirtioNetDevice(vmi, useEmulation) {
-		// Note that about network interface, useEmulation does not make
+	if util.NeedVirtioNetDevice(vmi, allowEmulation) {
+		// Note that about network interface, allowEmulation does not make
 		// any difference on eventual Domain xml, but uniformly making
 		// /dev/vhost-net unavailable and libvirt implicitly fallback
 		// to use QEMU userland NIC emulation.
@@ -1761,7 +1837,7 @@ func appendUniqueImagePullSecret(secrets []k8sv1.LocalObjectReference, newsecret
 	return append(secrets, newsecret)
 }
 
-// getMemoryOverhead computes the estimation of total
+// GetMemoryOverhead computes the estimation of total
 // memory needed for the domain to operate properly.
 // This includes the memory needed for the guest and memory
 // for Qemu and OS overhead.
@@ -1770,7 +1846,7 @@ func appendUniqueImagePullSecret(secrets []k8sv1.LocalObjectReference, newsecret
 //
 // Note: This is the best estimation we were able to come up with
 //       and is still not 100% accurate
-func getMemoryOverhead(vmi *v1.VirtualMachineInstance, cpuArch string) *resource.Quantity {
+func GetMemoryOverhead(vmi *v1.VirtualMachineInstance, cpuArch string) *resource.Quantity {
 	domain := vmi.Spec.Domain
 	vmiMemoryReq := domain.Resources.Requests.Memory()
 
@@ -1841,6 +1917,12 @@ func getMemoryOverhead(vmi *v1.VirtualMachineInstance, cpuArch string) *resource
 
 	addProbeOverheads(vmi, overhead)
 
+	// Consider memory overhead for SEV guests.
+	// Additional information can be found here: https://libvirt.org/kbase/launch_security_sev.html#memory
+	if util.IsSEVVMI(vmi) {
+		overhead.Add(resource.MustParse("256Mi"))
+	}
+
 	return overhead
 }
 
@@ -1966,6 +2048,7 @@ func getNetworkToResourceMap(virtClient kubecli.KubevirtClient, vmi *v1.VirtualM
 }
 
 func NewTemplateService(launcherImage string,
+	launcherQemuTimeout int,
 	virtShareDir string,
 	virtLibDir string,
 	ephemeralDiskDir string,
@@ -1980,6 +2063,7 @@ func NewTemplateService(launcherImage string,
 	precond.MustNotBeEmpty(launcherImage)
 	svc := templateService{
 		launcherImage:              launcherImage,
+		launcherQemuTimeout:        launcherQemuTimeout,
 		virtShareDir:               virtShareDir,
 		virtLibDir:                 virtLibDir,
 		ephemeralDiskDir:           ephemeralDiskDir,
@@ -2097,6 +2181,20 @@ func generatePodAnnotations(vmi *v1.VirtualMachineInstance) (map[string]string,
 	if HaveMasqueradeInterface(vmi.Spec.Domain.Devices.Interfaces) {
 		annotationsSet[ISTIO_KUBEVIRT_ANNOTATION] = "k6t-eth0"
 	}
+	annotationsSet[VELERO_PREBACKUP_HOOK_CONTAINER_ANNOTATION] = "compute"
+	annotationsSet[VELERO_PREBACKUP_HOOK_COMMAND_ANNOTATION] = fmt.Sprintf(
+		"[\"/usr/bin/virt-freezer\", \"--freeze\", \"--name\", \"%s\", \"--namespace\", \"%s\"]",
+		vmi.GetObjectMeta().GetName(),
+		vmi.GetObjectMeta().GetNamespace())
+	annotationsSet[VELERO_POSTBACKUP_HOOK_CONTAINER_ANNOTATION] = "compute"
+	annotationsSet[VELERO_POSTBACKUP_HOOK_COMMAND_ANNOTATION] = fmt.Sprintf(
+		"[\"/usr/bin/virt-freezer\", \"--unfreeze\", \"--name\", \"%s\", \"--namespace\", \"%s\"]",
+		vmi.GetObjectMeta().GetName(),
+		vmi.GetObjectMeta().GetNamespace())
+
+	// Set this annotation now to indicate that the newly created virt-launchers will use
+	// unix sockets as a transport for migration
+	annotationsSet[v1.MigrationTransportUnixAnnotation] = "true"
 	return annotationsSet, nil
 }
 
