@@ -37,10 +37,11 @@ import (
 	rbacv1 "k8s.io/api/rbac/v1"
 	extv1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/client-go/tools/record"
 	"k8s.io/client-go/util/workqueue"
 	apiregv1 "k8s.io/kube-aggregator/pkg/apis/apiregistration/v1"
 
-	v1 "kubevirt.io/client-go/api/v1"
+	v1 "kubevirt.io/api/core/v1"
 	"kubevirt.io/client-go/kubecli"
 	"kubevirt.io/client-go/log"
 	"kubevirt.io/kubevirt/pkg/certificates/triple"
@@ -67,7 +68,7 @@ func objectMatchesVersion(objectMeta *metav1.ObjectMeta, version, imageRegistry,
 
 	foundVersion, foundImageRegistry, foundID, _ := getInstallStrategyAnnotations(objectMeta)
 	foundGeneration, generationExists := objectMeta.Annotations[v1.KubeVirtGenerationAnnotation]
-	foundLabels := objectMeta.Labels[v1.ManagedByLabel] == v1.ManagedByLabelOperatorValue
+	foundLabels := util.IsManagedByOperator(objectMeta.Labels)
 	sGeneration := strconv.FormatInt(generation, 10)
 
 	if generationExists && foundGeneration != sGeneration {
@@ -94,6 +95,10 @@ func injectOperatorMetadata(kv *v1.KubeVirt, objectMeta *metav1.ObjectMeta, vers
 	}
 	objectMeta.Labels[v1.AppComponentLabel] = v1.AppComponent
 
+	if kv.Spec.ProductComponent != "" && util.IsValidLabel(kv.Spec.ProductComponent) {
+		objectMeta.Labels[v1.AppComponentLabel] = kv.Spec.ProductComponent
+	}
+
 	objectMeta.Labels[v1.ManagedByLabel] = v1.ManagedByLabelOperatorValue
 
 	if objectMeta.Annotations == nil {
@@ -431,9 +436,10 @@ type Reconciler struct {
 	clientset        kubecli.KubevirtClient
 	aggregatorclient install.APIServiceInterface
 	expectations     *util.Expectations
+	recorder         record.EventRecorder
 }
 
-func NewReconciler(kv *v1.KubeVirt, targetStrategy *install.Strategy, stores util.Stores, clientset kubecli.KubevirtClient, aggregatorclient install.APIServiceInterface, expectations *util.Expectations) (*Reconciler, error) {
+func NewReconciler(kv *v1.KubeVirt, targetStrategy *install.Strategy, stores util.Stores, clientset kubecli.KubevirtClient, aggregatorclient install.APIServiceInterface, expectations *util.Expectations, recorder record.EventRecorder) (*Reconciler, error) {
 	kvKey, err := controller.KeyFunc(kv)
 	if err != nil {
 		return nil, err
@@ -457,6 +463,7 @@ func NewReconciler(kv *v1.KubeVirt, targetStrategy *install.Strategy, stores uti
 		clientset,
 		aggregatorclient,
 		expectations,
+		recorder,
 	}, nil
 }
 
@@ -468,6 +475,9 @@ func (r *Reconciler) Sync(queue workqueue.RateLimitingInterface) (bool, error) {
 	if !util.IsValidLabel(r.kv.Spec.ProductName) {
 		log.Log.Errorf("invalid kubevirt.spec.productName: labels must be 63 characters or less, begin and end with alphanumeric characters, and contain only dot, hyphen or underscore")
 	}
+	if !util.IsValidLabel(r.kv.Spec.ProductComponent) {
+		log.Log.Errorf("invalid kubevirt.spec.productComponent: labels must be 63 characters or less, begin and end with alphanumeric characters, and contain only dot, hyphen or underscore")
+	}
 
 	targetVersion := r.kv.Status.TargetKubeVirtVersion
 	targetImageRegistry := r.kv.Status.TargetKubeVirtRegistry
@@ -570,7 +580,7 @@ func (r *Reconciler) Sync(queue workqueue.RateLimitingInterface) (bool, error) {
 	}
 
 	if shouldTakeUpdatePath(targetVersion, observedVersion) {
-		finished, err := r.updateKubeVirtSystem(daemonSetsRolledOver, controllerDeploymentsRolledOver)
+		finished, err := r.updateKubeVirtSystem(controllerDeploymentsRolledOver)
 		if !finished || err != nil {
 			return false, err
 		}
@@ -617,7 +627,7 @@ func (r *Reconciler) createOrRollBackSystem(apiDeploymentsRolledOver bool) (bool
 	// create/update API Deployments
 	for _, deployment := range r.targetStrategy.ApiDeployments() {
 		deployment := deployment.DeepCopy()
-		err := r.syncDeployment(deployment)
+		deployment, err := r.syncDeployment(deployment)
 		if err != nil {
 			return false, err
 		}
@@ -635,7 +645,7 @@ func (r *Reconciler) createOrRollBackSystem(apiDeploymentsRolledOver bool) (bool
 
 	// create/update Controller Deployments
 	for _, deployment := range r.targetStrategy.ControllerDeployments() {
-		err := r.syncDeployment(deployment)
+		deployment, err := r.syncDeployment(deployment)
 		if err != nil {
 			return false, err
 		}
@@ -647,8 +657,8 @@ func (r *Reconciler) createOrRollBackSystem(apiDeploymentsRolledOver bool) (bool
 
 	// create/update Daemonsets
 	for _, daemonSet := range r.targetStrategy.DaemonSets() {
-		err := r.syncDaemonSet(daemonSet)
-		if err != nil {
+		finished, err := r.syncDaemonSet(daemonSet)
+		if !finished || err != nil {
 			return false, err
 		}
 	}
